#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1e.h"
#include "m_TreeNode1b.h"



void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs > 2)
      mexErrMsgTxt("1 or 2 input arguments are required");
   if (nlhs>2)
      mexErrMsgTxt("Too many outputs");

   int PIXVALS_IN = 0;
   int SZRNG_IN = 1;

   int rows = mxGetM(prhs[PIXVALS_IN]);
   int cols = mxGetN(prhs[PIXVALS_IN]);

   int Output = 0;

   /* Get the pointers to the input Data */
   double *pixValsPtr = mxGetPr(prhs[PIXVALS_IN]);

   double **pixVals = buildMatrix<double>(rows, cols);
   for (int j=0; j < cols; j++){
      for (int i=0; i < rows; i++){
         pixVals[i][j] = *pixValsPtr;
         pixValsPtr++;
      }
   }

   if (Output){
      printf("rows: %d, cols: %d\n", rows, cols);
   }

   Window bestBB;
   if (nrhs == 1){
      bestBB = findBox1b2(pixVals, rows, cols);
   } else if (nrhs == 2){
      double *szRngPtr = mxGetPr(prhs[SZRNG_IN]);
      int *szRng = buildVector<int>(4);
      for (int k=0; k < 4; k++) szRng[k] = (int) szRngPtr[k];
      SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);
      if (Output){
         printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n",
            szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
      }
      bestBB = findBox1e2(pixVals, rows, cols, szRange);
      destroyVector<int>(szRng, 4);
   }

   /*if (!isImAssignGood(ImAssign, rows, cols)){
      mexErrMsgTxt("ImAssign, 1st argument must be matrix of positive integers");
   }*/

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);

   //If require the energy
   if (nlhs >= 2){
      double ***integralIms = cmpIntegralIms4(pixVals, rows, cols);
      double *hist1 = bestBB.getPosNegScore(integralIms);
      double energy = hist1[0] - hist1[1];
      plhs[1] = mxCreateDoubleScalar(energy);
      destroyArrayThree<double>(integralIms, rows, cols, 2);
      destroyVector<double>(hist1, 2);
   }

   destroyMatrix<double>(pixVals, rows, cols);
}
