#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1f.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){   
   /* check for the proper no. of input and outputs */
   if (nrhs != 8)
      mexErrMsgTxt("8 input arguments are required");
   if (nlhs>2)
      mexErrMsgTxt("Too many outputs");

   int nKeyPnt = mxGetM(prhs[0]);
   int rows = (int) *mxGetPr(prhs[1]);
   int cols = (int) *mxGetPr(prhs[2]);
   
   int histSize = mxGetM(prhs[3]);
   int m = mxGetN(prhs[3]);

   int Output = 0;
   
   /* Get the pointers to the input Data */  
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *pyrWeightsPtr = mxGetPr(prhs[3]);       
   double *branchPtr = mxGetPr(prhs[4]);
   double *levelPtr = mxGetPr(prhs[5]);
   double *szRngPtr = mxGetPr(prhs[6]);
   double *tolFacPtr = mxGetPr(prhs[7]);
   
   /* create matrices */
   int **ImAssign, *szRng;
   double **pyrWeights;

   
   int nBranch = (int) *branchPtr;
   int nLevel = (int) *levelPtr;
   double tolFac = (double) *tolFacPtr;
   
   

   ImAssign = buildMatrix<int>(nKeyPnt, 3); 
   pyrWeights = buildMatrix<double>(m, histSize);
   szRng = buildVector<int>(4);    


   // Assign the data for: ImAssign
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;  
      }
   }

   
   // Assign the data for pyrWeights
   for (int j=0; j < m; j++){
      for (int i=0; i < histSize; i++){
         pyrWeights[j][i] = (double) *pyrWeightsPtr;
         pyrWeightsPtr++;
      }
   }

   // size range
   for (int k=0; k < 4; k++){
      szRng[k] = (int) (*szRngPtr);
      szRngPtr++;
   }

   SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);

   /*if (!isImAssignGood(ImAssign, rows, cols)){
      mexErrMsgTxt("ImAssign, 1st agrument must be matrix of positive integers");
   }*/

   
   if (Output){
      printf("rows: %d, cols: %d, histSize: %d \n", rows, cols, histSize);
      printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n", 
         szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
   }   
  
   double energy_best;
   Window bestBB = findBox1f(ImAssign, nKeyPnt, rows, cols, 
      pyrWeights, histSize, nBranch, nLevel, szRange, tolFac, &energy_best);


   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);   
   double *labelOutPtr = mxGetPr(plhs[0]); 
   
   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);
   
      //If require the energy
   if (nlhs >= 2){      
      plhs[1] = mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
      double *energyPtr = mxGetPr(plhs[1]);
      energyPtr[0] = energy_best;      
   }

   // delete data
   destroyMatrix<int>(ImAssign, nKeyPnt, 3);
   destroyMatrix<double>(pyrWeights, m, histSize);   
   destroyVector<int>(szRng, 4);
} 
