#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1g.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 3)
      mexErrMsgTxt("3 input arguments are required");
   if (nlhs > 3)
      mexErrMsgTxt("Too many outputs");

   int PIXVALS_IN = 0;
   int GTWIN_IN   = 1;
   int SZRNG_IN   = 2;

   int rows = mxGetM(prhs[PIXVALS_IN]);
   int cols = mxGetN(prhs[PIXVALS_IN]);

   int Output = 0;

   /* Get the pointers to the input Data */
   double *pixValsPtr = mxGetPr(prhs[PIXVALS_IN]);
   double **pixVals = buildMatrix<double>(rows, cols);
   for (int j=0; j < cols; j++){
      for (int i=0; i < rows; i++){
         pixVals[i][j] = *pixValsPtr;
         pixValsPtr++;
      }
   }

   double *gtBoxPtr = mxGetPr(prhs[GTWIN_IN]);
   int *gtBox = buildVector<int>(4);
   for (int k=0; k < 4; k++) gtBox[k] = (int) gtBoxPtr[k];
   Window gtWin(gtBox[0]-1, gtBox[1]-1, gtBox[2]-1, gtBox[3]-1);

   double *szRngPtr = mxGetPr(prhs[SZRNG_IN]);
   int *szRng = buildVector<int>(4);
   for (int k=0; k < 4; k++) szRng[k] = (int) szRngPtr[k];
   SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);

   if (Output){
      printf("rows: %d, cols: %d\n", rows, cols);
      printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n",
         szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
      printf("GT Box: l: %d, t: %d, r: %d, b: %d\n",
         gtWin.ul_x, gtWin.ul_y, gtWin.lr_x, gtWin.lr_y);
   }

   Window bestBB = findBox1g2(pixVals, rows, cols, gtWin, szRange);
   /* Create the outGoing Array  */
   plhs[0] = mxCreateDoubleMatrix(4, 1, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);

   if (nlhs > 1){ // return energy
       double ***integralIms = cmpIntegralIms4(pixVals, rows, cols);
       double *hist1 = bestBB.getPosNegScore(integralIms);
       double energy = hist1[0] - hist1[1];
       plhs[1] = mxCreateDoubleScalar(energy);
       destroyVector<double>(hist1, 2);
       destroyArrayThree<double>(integralIms, rows, cols, 2);
   }

   if (nlhs > 2){ // return hitrate
      Window inter1 = bestBB.intersectWin(gtWin);
      double minArea = (double)inter1.getArea();
      Window union1 = bestBB.unionWin(gtWin);
      double maxArea = (double)union1.getArea();
      double hitRateE = 0;
      if (maxArea > 0) hitRateE = minArea/maxArea;
      plhs[2] = mxCreateDoubleScalar(hitRateE);
   }

   destroyVector<int>(szRng, 4);
   destroyVector<int>(gtBox, 4);
   destroyMatrix<double>(pixVals, rows, cols);
}
