/*
 * m_mexFindBox1i.cpp
 *
 * Created on: May 9, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1i.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 8)
      mexErrMsgTxt("8 input arguments are required");
   if (nlhs>2)
      mexErrMsgTxt("Too many outputs");


   int nKeyPnt = mxGetM(prhs[0]);
   int rows = (int) *mxGetPr(prhs[1]);
   int cols = (int) *mxGetPr(prhs[2]);

   int histSize = mxGetM(prhs[3]);

   int Output = 1;

   /* Get the pointers to the input Data */
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *weightsPtr = mxGetPr(prhs[3]);
   double *targetPtr = mxGetPr(prhs[4]);
   double C = mxGetScalar(prhs[5]);
   double *szRngPtr = mxGetPr(prhs[6]);
   double tolFac = mxGetScalar(prhs[7]);



   /* create matrices */
   int **ImAssign = buildMatrix<int>(nKeyPnt, 3);
   double *weights = buildVector<double>(histSize);
   int *target = buildVector<int>(histSize);
   int *szRng = buildVector<int>(4);

   // Assign the data to weights and target histogram
   for (int k=0; k<histSize; k++){
      weights[k] = (double) (*weightsPtr);
      weightsPtr++;
      target[k] = (int)(*targetPtr);
      targetPtr++;
   }

   // Assign the data for: ImAssign
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;
      }
   }

   for (int k=0; k < 4; k++){
      szRng[k] = (int) (*szRngPtr);
      szRngPtr++;
   }

   SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);

   if (Output){
      printf("rows: %d, cols: %d, histSize: %d, nKeyPnt: %d \n",
              rows, cols, histSize, nKeyPnt);
      printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d, C: %f, tolFac: %f\n",
            szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax, C, tolFac);
   }

   Window bestBB = findBox1i(ImAssign, nKeyPnt, rows, cols, weights, histSize,
         target, C, szRange, tolFac);


   if (Output){
      printf("Finish calling findBox1i\n");
   }

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);

   //If require the energy
   if (nlhs >= 2){
      int ***integralIms = cmpIntegralIms1(ImAssign, nKeyPnt, rows, cols, histSize);
      int *hist = bestBB.getHist(integralIms, histSize);

      double energy = TreeNode1i::cmpEnergy(hist, target, weights, histSize, bestBB, C);
      plhs[1] = mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
      double *energyPtr = mxGetPr(plhs[1]);
      energyPtr[0] = energy;
      destroyArrayThree<int>(integralIms, rows, cols, histSize);
      destroyVector<int>(hist, histSize);
   }

   destroyMatrix<int>(ImAssign, nKeyPnt, 3);
   destroyVector<double>(weights, histSize);
   destroyVector<int>(target, histSize);
   destroyVector<int>(szRng, 4);
}
