/*
 * m_mexMultiLabelKSeg.cpp
 *
 * Created on: May 12, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_multiLabelKSeg.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 3)
      mexErrMsgTxt("3 input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");

   int A_IN   = 0;
   int NSEG_IN = 1;
   int SEG_SWCH_IND = 2;

   int len = mxGetM(prhs[A_IN]);
   int nLabel = mxGetN(prhs[A_IN]);
   int nSeg = (int)mxGetScalar(prhs[NSEG_IN]);

   bool Output = false;

   /* Get the pointers to the input Data */
   double *APtr = mxGetPr(prhs[A_IN]);
   double **A = buildMatrix<double>(nLabel, len);
   for (int j = 0; j < nLabel; j++){
      for (int i = 0; i < len; i++) A[j][i] = (double)(*APtr++);
   }
   bool **segSwchInd = buildMatrix<bool>(nLabel, nLabel);
   double *segSwchIndPtr = mxGetPr(prhs[SEG_SWCH_IND]);
   for (int j = 0; j < nLabel; j++){
      for (int i = 0; i < nLabel; i++) segSwchInd[i][j] = (bool)(*segSwchIndPtr++);
   }

   if (Output) mexPrintf("nLabel: %d, nSeg: %d, len: %d\n", nLabel, nSeg, len);

   double energy;
   int *rslt = multiLabelKseg2(A, nLabel, len, nSeg, segSwchInd, &energy);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(len, 1, mxDOUBLE_CLASS, mxREAL);
   double *assgn = mxGetPr(plhs[0]);
   for (int j = 0; j < len; j++) *assgn++ = rslt[j] + 1;

   plhs[1] = mxCreateDoubleScalar(energy);

   destroyMatrix<double>(A, nLabel, len);
   destroyMatrix<bool>(segSwchInd, nLabel, nLabel);
}
