function [label, energy] = ml_findMulti1DSegs3(pixVals, lmin, lmax)
% function [label, energy] = ml_findMulti1DSegs3(pixVals, lmin, lmax)
% Divide a time series into segments that maximize MMC score.
% Roughly speaking it seeks a division that maximizes the difference between the best 
% and the second best class, summing across all resulting segments.
% The score of a segment [a,b] belonging to class k^th is sum_{i=a,...,b} {pixVals(i,k)}
% This function find GLOBAL minimum using dynamic programming
% Inputs:
%   pixVals: a imH*nLabel matrix
%   lmin, lmax: the minimum and maximum lengths of each resulting segment. 
% Output:
%   label: 3*nSeg matrix label(:,i) is [start, end, label]' of segment i^th.
% Examples:
%   pixVals = [0.1 0.1 0.1 0.1 0.1   0   0   0; ...
%                0   0 0.2 0.2   0   0   0   0; ...
%                0   0   0   0 0.3 0.3 0.3 0.3]';
%   [label, energy] = ml_findMulti1DSegs3(pixVals, 2,3);
%   [label, energy] = ml_findMulti1DSegs3(pixVals, 3,3);
%   Eg2:
%   pixVals = zeros(1000,4);
%   pixVals(1:1000,1) = 0.0001;
%   pixVals(300:700,2) = 0.0002;
%   pixVals(701:1000,4) = 0.0003;
%   [label, energy] = ml_findMulti1DSegs3(pixVals, 1, 400);
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 10 Nov 10

if (lmin < 1)
    error('lmin should be at least 1');
end;

if (lmin > lmax)
    error('lmin should not be greater than lmax');
end;

[label, energy] = m_mexFindMulti1DSegs3(pixVals, lmin, lmax);
if isinf(energy) 
    label = [];
end
    