#ifndef M_MULTI1DSEGS_H
#define M_MULTI1DSEGS_H
#include "m_TreeNode1.h"



//Similar to TreeNode1b but to find maximum instead of minimum
class TreeNode1b_2:public TreeNode1{
public:
   static double ***integralIms;
   TreeNode1b_2(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1b_2():TreeNode1(){};
   void cmpBnds();
};

class Interval{
public:
   static double *integralIm;
   Interval():l(-1), u(-1), sL(-1), sU(-1){};
   Interval(int l_, int u_):l(l_), u(u_), sL(-1), sU(-1){};
   Interval(int l_, int u_, bool isChosen_):
      l(l_), u(u_), sL(-1), sU(-1), isChosen(isChosen_){};
   int l, u; // should be non negative numbers
   int sL, sU; // lower and upper of best sub interval.
   double val; //value of the best sub interval
   bool isChosen; // 1: chosen segment, 0: not chosen segment

   void findBestSubInt();
   void findBestSubInt2();

   //split the interval, the middle interval is the bestSubInt
   void splitInt(Interval *left, Interval *right);

};

// Function for comparing two intervals
class IntervalCompare{
public:
   bool operator() (const Interval& lhs, const Interval& rhs){
      return (lhs.val > rhs.val);
   };
};



class Multi1DSegs{
public:
   int nSeg, length;
   double **segs;
   double energy;
   Multi1DSegs(){
      nSeg = 0;
      energy = 0;
      length = 0;
   };
   Multi1DSegs(double **segs_, int nSeg_, double energy_, int length_){
      segs = segs_;
      nSeg = nSeg_;
      energy = energy_;
      length = length_;
   }

   void copy(Multi1DSegs *newSegs);

   ~Multi1DSegs();
};


/**
 * This function is more than a helper function for findMulti1DSegs.
 * It can be used to test the feasibility of a problem; this allows early stopping
 * \param length the length of the signal
 * \param integralIms the integral images
 * \param nSeg number of maximum disjoint segments
 * \param geVal good enough value. The function terminates when the total energy
 *    found is lower than this value. If this criterion is never met, the algorithm
 *    stops when the number of segments reaches nSeg.
 * \return # of intervals and their ends.
 */
Multi1DSegs findMulti1DSegs_helper2(int & length, double ***& integralIms, int nSeg, double geVal);



/**
 * Find the best nSeg segments of a 1D signal that minimizes:
 *    sum_i {weights(i)*hist(i)}
 * This function finds the GLOBAL minimum!
 * Inputs:
 *    kpAssign: a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0] = 1 kpAssign[i][1], kpAssign[i][2] are col#, bin#
 *       entries are positive integers.
 *    nKeyPnt: # of keypoints.
 *    imW: the length of the signal
 *    weights: a 1-dim array for weights for histogram bins
 *    nHistBin: # of histogram bins
 *    nSeg: # of maximum segments to extract.
 * Output:
 *    nSeg disjoint segments that minimizes the desired energy.
 *    K <= nSeg.
 */
double** findMulti1DSegs(int **kpAssign, int nKeyPnt, int imW, double *weights,
     int nHistBin, int nSeg,int *nChosen);

/**
 * Find the best nSeg segments of a 1D signal that minimizes the sum of
 *    elements that they contain.
 * This function finds the GLOBAL minimum!
 * Inputs:
 *    pixVals: a 1D array for values.
 *    imW: the length of the signal
 *    nSeg: # of maximum segments to extract.
 * Output:
 *    A K disjoint segments that minimizes the desired energy K <= nSeg.
 *    K is returned in *nChosen
 * See also:
 *    findMulti1DSegs which serves the same purpose but use different input format
 */
double** findMulti1DSegs2(double *pixVals, int imW, int nSeg, int *nChosen);

/**
 * Find the best nSeg segments of a 1D signal that mimimizes:
 *    sum_i {weights(i)*hist(i)}
 * This function finds the segments sequentially.
 * The global minimum might not be achieved.
 * Although this function does not find global minimum. It
 * can incorporate the size range in the constraints.
 * Inputs:
 *    kpAssign: a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0] = 1 kpAssign[i][1], kpAssign[i][2] are col#, bin#
 *       entries are positive integers.
 *    nKeyPnt: # of keypoints.
 *    imW: the length of the signal
 *    weights: a 1-dim array for weights for histogram bins
 *    nHistBin: # of histogram bins
 *    nSeg: # of maximum segments to extract. If nSeg = -1, this
 *       will extract as many segments as possible until no more segment
 *       with value <= thres remains.
 *    szRange: the range of the segments.
 *    thres: all the chosen segments must have value <= thres.
 * Output:
 *    A K disjoint segments that minimizes the desired energy.
 *       K <= nSeg.
 */
double** findMulti1DSegs_seq(int **kpAssign, int nKeyPnt, int imW, double *weights,
      int nHistBin, int nSeg, SizeRange szRange, double thres,
      int *nChosen);

/**
 * Find the best nSeg segments of a 1D signal that minimizes the sum of
 *    elements that they contain.
 * This function finds the segments sequentially.
 * The global minimum might not be achieved.
 * Although this function does not find global minimum. It
 * can incorporate the size range in the constraints.
 * Inputs:
 *    pixVals: a 1D array for values.
 *    imW: the length of the signal
 *    nSeg: # of maximum segments to extract. If nSeg = -1, this
 *       will extract as many segments as possible until no more segment
 *       with value <= thres remains.
 *    szRange: the range of the segments.
 *    thres: all the chosen segments must have value <= thres.
 * Output:
 *    A K disjoint segments that minimizes the desired energy, K <= nSeg.
 * See also:
 *    findMulti1DSegs_seq serves the same purpose but use different input format
 *    findMulti1DSegs, findMulti1DSegs2
 */
double** findMulti1DSegs_seq2(double *pixVals, int imW, int nSeg,
      SizeRange szRange, double thres, int *nChosen);


/*Get TP, FP for by varying number of segments*/
int **getTPFPsFor4allNSegs(double *pixVals, int length, bool *gtLabel);


#endif
