/*
 * m_NormMulti1DSegs_tests.cpp
 *
 * Created on: Aug 13, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include <iostream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include <fstream>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_NormMulti1DSegs.h"
#include "m_tests.h"


class Multi1DSegsTest3{
public:
   double *pixVals, alpha, beta, expVal;
   int length, nSeg, **expSegs, nChosenSeg;

   Multi1DSegsTest3(int length_, int nSeg_, double alpha_, double beta_){
      length = length_;
      nSeg = nSeg_;
      alpha = alpha_;
      beta = beta_;
      pixVals = new double[length];
   };

   void setVal(int l, int u, double val){ //assign histogram bin number
      for (int i=l; i <= u; i++) pixVals[i] = val;
   };

   //Set the expected result
   void setExpSegs(int *expSegs_, int nChosenSeg_){
      nChosenSeg = nChosenSeg_;
      int len = 0;
      if (nChosenSeg > 0){
         expSegs = buildMatrix<int>(2, nChosenSeg);
         expVal = 0; //expected energy
         for (int i=0; i < nChosenSeg; i++){
            expSegs[0][i] = expSegs_[2*i];
            expSegs[1][i] = expSegs_[2*i+1];
            len += expSegs[1][i] - expSegs[0][i] + 1;
            double ***integralIms = cmpIntegralIms4(&pixVals, 1, length);
            Window win(expSegs[0][i], 0, expSegs[1][i], 0);
            double *hist = win.getPosNegScore(integralIms);
            expVal += hist[0] - hist[1];
            destroyArrayThree<double>(integralIms, 1, length, 2);
            destroyVector<double>(hist, 2);
         }
      }
      expVal = (expVal + alpha)/(len + beta);
   };

   ~Multi1DSegsTest3(){
      delete [] pixVals;
      if (nChosenSeg > 0 ) destroyMatrix<int>(expSegs, 2, nChosenSeg);
   };
};

void testNormMulti1DSegs(){
   cout << "testNormMulti1DSegs() " << endl;
   queue<Multi1DSegsTest3*> tests;
   int length = 20;
   int nSeg = 2;
   double alpha = 0;
   double beta = 2;
   double tol = 0.0001;

   Multi1DSegsTest3 test1(length, nSeg, alpha, beta);
   test1.setVal(0, length-1, 1);
   test1.expVal = alpha/beta;
   test1.nChosenSeg = 0;



   Multi1DSegsTest3 test2(length, nSeg, alpha, beta);
   test2.setVal(0, length-1, -1);
   int expSegs2[] = {0, 19};
   test2.setExpSegs(expSegs2, 1);


   Multi1DSegsTest3 test3(length, nSeg, alpha, beta);
   test3.setVal(0, length-1, 1);
   test3.setVal(1, 5, -1);
   test3.setVal(11, 13, -1);
   int expSegs3[] = {1, 5, 11, 13};
   test3.setExpSegs(expSegs3, 2);

   Multi1DSegsTest3 test4(length, nSeg, alpha, beta);
   test4.setVal(0, length-1, 1);
   test4.setVal(1, 5, -1);
   test4.setVal(11, 13, -0.3);
   int expSegs4[] = {1, 5};
   test4.setExpSegs(expSegs4, 1);

   Multi1DSegsTest3 test5(length, nSeg, alpha, 15);
   test5.setVal(0, length-1, 1);
   test5.setVal(1, 5, -1);
   test5.setVal(11, 13, -0.3);
   int expSegs5[] = {1, 5, 11, 13};
   test5.setExpSegs(expSegs5, 2);

   Multi1DSegsTest3 test6(length, 1, alpha, 15);
   test6.setVal(0, length-1, 1);
   test6.setVal(1, 5, -1);
   test6.setVal(11, 13, -0.3);
   int expSegs6[] = {1, 5};
   test6.setExpSegs(expSegs6, 1);

   Multi1DSegsTest3 test7(length, 1, alpha, 100);
   test7.setVal(0, length-1, 0.001);
   test7.setVal(1, 5, -1);
   test7.setVal(11, 13, -0.3);
   int expSegs7[] = {1, 13};
   test7.setExpSegs(expSegs7, 1);

   Multi1DSegsTest3 test8(length, nSeg, alpha, 100);
   test8.setVal(0, length-1, 0.001);
   test8.setVal(1, 5, -1);
   test8.setVal(11, 13, -0.3);
   int expSegs8[] = {1, 5, 11, 13};
   test8.setExpSegs(expSegs8, 2);

   Multi1DSegsTest3 test9(length, 3, alpha, 100);
   test9.setVal(0, length-1, 0.001);
   test9.setVal(1, 5, -1);
   test9.setVal(11, 13, -0.3);
   int expSegs9[] = {1, 5, 11, 13};
   test9.setExpSegs(expSegs9, 2);

   Multi1DSegsTest3 test10(length, 3, -1, 100);
   test10.setVal(0, length-1, 0.001);
   test10.setVal(1, 5, -1);
   test10.setVal(11, 13, -0.3);
   int expSegs10[] = {1, 5, 11, 13};
   test10.setExpSegs(expSegs10, 2);

   Multi1DSegsTest3 test11(length, 3, -99, 100);
   test11.setVal(0, length-1, 0.001);
   test11.setVal(1, 5, -1);
   test11.setVal(11, 13, -0.3);
   int expSegs11[] = {1, 5};
   test11.setExpSegs(expSegs11, 1);

   Multi1DSegsTest3 test12(length, 3, 0, 15);
   test12.setVal(0, length-1, 1);
   test12.setVal(1, 5, -1);
   test12.setVal(11, 14, -0.25);
   int expSegs12[] = {1, 5, 11, 14};
   test12.setExpSegs(expSegs12, 2);

   Multi1DSegsTest3 test13(length, 3, 0, 15.1);
   test13.setVal(0, length-1, 1);
   test13.setVal(1, 5, -1);
   test13.setVal(11, 14, -0.25);
   int expSegs13[] = {1, 5, 11, 14};
   test13.setExpSegs(expSegs13, 2);

   Multi1DSegsTest3 test14(length, 3, 0, 14.9);
   test14.setVal(0, length-1, 1);
   test14.setVal(1, 5, -1);
   test14.setVal(11, 14, -0.25);
   int expSegs14[] = {1, 5};
   test14.setExpSegs(expSegs14, 1);


   tests.push(&test1);
   tests.push(&test2);
   tests.push(&test3);
   tests.push(&test4);
   tests.push(&test5);
   tests.push(&test6);
   tests.push(&test7);
   tests.push(&test8);
   tests.push(&test9);
   tests.push(&test10);
   tests.push(&test11);
   tests.push(&test12);
   tests.push(&test13);
   tests.push(&test14);


   int i= 0;
   while (!tests.empty()){
      Multi1DSegsTest3 *test = tests.front();
      tests.pop();
      i++; cout << "==============Testing NormMulti1DSegs " << i << endl;
      for (int k=0; k < test->length; k++){
         cout << " " << test->pixVals[k];
      }
      cout << endl;

      NormMulti1DSegs rslt = findNormMulti1DSegs(test->pixVals, test->length, test->nSeg, test->alpha, test->beta, tol);
      printf("  # of iterations: %d\n", rslt.nIter);

      cout << "  # selected ints: " << rslt.nSeg << " ==> ";

      for (int k=0; k < rslt.nSeg; k++){
         cout << "[" << rslt.segs[0][k] << ", "
              << rslt.segs[1][k] << "]; " ;
      }
      cout << endl;

      double bestRetVal = (rslt.energy + test->alpha)/(rslt.length + test->beta);

      if (bestRetVal < test->expVal - tol){
         cout << "WARNING: expected energy is bigger than returned energy" << endl;
         cout << bestRetVal << " " << test->expVal << endl;
      } else if (bestRetVal > test->expVal + EPS){
         cout << "FAILED, expected E: " << test->expVal << ", return: " << bestRetVal << endl;
         cout << "  expected: ";
         for (int k = 0; k < test->nChosenSeg; k++){
            cout << "[" << test->expSegs[0][k] << ", "
               << test->expSegs[1][k] << "]; ";
         }
         cout << endl;
      } else {
         cout << "PASSED" << endl;
      }
   }
}

