#ifndef M_TREENODE1A_H
#define M_TREENODE1A_H

#include "m_TreeNode1.h"

class TreeNode1a:public TreeNode1{
public:
   static int ***integralIms;
   static int *hist_target;
   static double *binWeights;
   static int nHistBin;
   static double C;

   TreeNode1a(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1a():TreeNode1(){};
   void cmpBnds();
};


/**
 * Find the best box in an image matching the given target histogram
 *    by minimizing the objective:
 *    sum_i {weights(i)*[|hist(i) - hist_target(i)| - C*(hist(i) + hist_target(i)]}
 * Inputs:
 *    imAssign: a 2 dim array for histogram bin assignments, entries should be
 *       positive integers.
 *    imH, imW: height and width of the image
 *    hist_target: a 1-dim array for target histogram
 *    weights: a 1-dim array for weights for histogram bins
 *    nHistBin: # of histogram bins
 *    C: tradeoff b/t having good match to the target histogram and the size of the box
 *    tolFactor: tolerance factor for stopping criteria of branch and bound
 *       if tolFactor = 0, run BB until global optimality is known.
 *       the bigger tolFactor, the earlier BB terminates.
 *    knownBest: the solution is known to be better than this value.
 *       branches with the lower bound less than this value is not explored.
 *       if no leaf node has energy smaller than this value, the method
 *       return an arbitrary solution.
 * Output:
 *    A best (within tolerance) rectangle that minimizes the desired energy
 */
Window findBox1a(int **imAssign, int imH, int imW,
               int *hist_target, double *weights,
               int nHistBin, double C, double tolFactor, double knownBest);
Window findBox1a_helper(int ***integralIms, TreeNode1a rootNode,
                      int *hist_target, double *weights,
                      int nHistBin, double C, double tolFactor, double knownBest);


#endif
