#ifndef M_TREENODE1B_H
#define M_TREENODE1B_H
#include "m_TreeNode1.h"

/**
 *
 */
class TreeNode1b:public TreeNode1{
public:
   static double ***integralIms;
   TreeNode1b(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1b():TreeNode1(){};
   void cmpBnds();
};

/**
 * Find the best box in an image that minimizes:
 *    \f$ \sum_i {weights(i)\times hist(i)} \f$
 * This function can be used to find the largest enclosing box.
 * \param kpAssign a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 * \param nKeyPnt # of keypoints.
 * \param imH, imW height and width of the image
 * \param weights a 1-dim array for weights for histogram bins
 * \param nHistBin # of histogram bins
 * \return The best rectangle that minimizes the desired energy
 * \see findBox1b2 serves the same purpose, but uses different input format.
 * \see findBox1e2 can incoroprate size range.
 */
Window findBox1b(int **kpAssign, int nKeyPnt, int imH, int imW,
                double *weights, int nHistBin);

/**
 * Find the best box in an image that has the sum of enclosing elements minimized.
 * \f$\sum_{(i,j) \in \textrm{win}} \textrm{pixVals}(i,j)\f$
 * \param pixVals a imH*imW 2D array of scalars.
 * \param imH, imW height and width of the image
 * \return The best rectangle that minimizes the desired energy.
 * \see findBox1b serves similar purpose, but uses different input format.
 *    This function has pre-multiplied the weights and histogram bins.
 * \see findBox1e2 accepts similar input format except for the szRange.
 *    This function is faster than findBox1e2 if szRange is not necessary
 */
Window findBox1b2(double **pixVals, int imH, int imW);

#endif
