#ifndef M_TREENODE1C_H
#define M_TREENODE1C_H
#include <vector>
#include "m_TreeNode1.h"

class HistTree{
public:
   int nBranch, sqrtBranch, nLevel, *hist, nHistBin;
   bool isLeaf;
   double weight;
   double emptyWinE; // energy of empty window.
   double *levelWeights;
   vector<HistTree*> branches;

   // Constructors
   HistTree(int **hists, int nBranch, int nLevel, int nHistBin, double *weights);
   HistTree(){};

   //Destructor
   ~HistTree();

   //functions
   double cmpLb(WindowPair winPair, int ***integralIms, double *binWeights, int nHistBin, double C);


   double cmpE(Window win, int ***integralIms, double *binWeights, int nHistBin, double C);

   void setEmptyWinE(double *binWeights);
protected:
   double cmpLb1(WindowPair winPair, int ***integralIms, double *binWeights, int nHistBin, double C);
   double cmpLb2(WindowPair winPair, int ***integralIms, double *binWeights, int nHistBin, double C);
   void cmpLb2_helper(WindowPair winPair, int ***integralIms, double *binWeights,
                       int nHistBin, double *levelLbs);

};

class TreeNode1c:public TreeNode1{
public:
   static int ***integralIms;
   static HistTree *targetHists;
   static double *binWeights;
   static int nHistBin;
   static double C;

   int sqrtBrnch;
   TreeNode1c(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1c():TreeNode1(){};

   void cmpBnds();
};


/**
 * Find the best box in an image matching the pyramid of histograms
 *    like in the pyramid matching kernel.
 *    by minimizing the objective:
 *    sum_i {weights(i)*[sum_j levelWeights(j)*|hist_j(i) - hist_target_j(i)|
 *           - C*(hist_1(i) + hist_target_1(i)]}
 * Inputs:
 *    imAssign: a 2 dim array for histogram bin assignments, entries should be
 *       positive integers.
 *    imH, imW: height and width of the image
 *    targetHists: a HistTree for target pyramid of histograms
 *    weights: a 1-dim array for weights for histogram bins
 *    nHistBin: # of histogram bins
 *    C: tradeoff b/t having good match to the target histogram and the size of the box
 *    tolFactor: tolerance factor for stopping criteria of branch and bound
 *       if tolFactor = 0, run BB until global optimality is known.
 *       the bigger tolFactor, the earlier BB terminates.
 *    knownBest: the solution is known to be better than this value.
 *       branches with the lower bound less than this value is not explored.
 *       if no leaf node has energy smaller than this value, the method
 *       return an arbitrary solution.
 *    energy: the energy of the returned output.
 * Output:
 *    A best (within tolerance) rectangle that matches the pyramid of histograms.
 */
Window findBox1c(int **imAssign, int imH, int imW,
         HistTree *targetHists, double *weights,
         int nHistBin, double C, double tolFac, double knownBest, double *energy);

#endif
