#ifndef M_TREENODE1E_H
#define M_TREENODE1E_H
#include "m_TreeNode1.h"

class TreeNode1e:public TreeNode1{
public:
   static double ***integralIms;
   static SizeRange szRange;
   Window ubWin;

   TreeNode1e(WindowPair winPair_):TreeNode1(winPair_){};
   TreeNode1e():TreeNode1(){};
   void cmpBnds();

};

/**
 * Find the best box in an image that minimizes:
 *    \f$ \sum_i {weights(i)\times hist(i)} \f$
 * This function can be used to find the largest enclosing box.
 * \param kpAssign a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 * \param nKeyPnt # of keypoints.
 * \param imH, imW height and width of the image
 * \param weights a 1-dim array for weights for histogram bins
 * \param nHistBin # of histogram bins
 * \param szRange range for width and height of the box
 * \return The best rectangle that minimizes the desired energy
 * \see findBox1e2 serves the same purpose, but uses different input format.
 * \see findBox1b faster if no size range is necessary.
 */
Window findBox1e(int **kpAssign, int nKeyPnt, int imH, int imW,
                 double *weights, int nHistBin,
                 SizeRange szRange);


/**
 * Find the best box in an image that has the sum of enclosing elements minimized.
 * \f$\sum_{(i,j) \in \textrm{win}} \textrm{pixVals}(i,j)\f$
 * \param pixVals a imH*imW 2D array of scalars.
 * \param imH, imW height and width of the image
 * \return The best rectangle that minimizes the desired energy.
 * \see findBox1e serves similar purpose, but uses different input format.
 * \see findBox1b2 accepts similar input format but cannot constrain size range
 *    If size range is not necessary, use findBox1b2
 */
Window findBox1e2(double **pixVals, int imH, int imW, SizeRange szRange);

#endif
