/*
 * m_TreeNode1i.cpp
 *
 *  Created on: May 8, 2009
 *      Author: Minh Hoai Nguyen
 */

#include "m_malloc.h"
#include "m_utils.h"
#include "m_BranchBound.cpp"
#include "m_TreeNode1i.h"


double
TreeNode1i::cmpEnergy(int *hist, int *target, double *weights, int nHistBin,
      Window win, double C){
   double energy = 0;
   for (int i=0; i < nHistBin; i++){
      energy += weights[i]*pow((double) (hist[i] - target[i]), 2);
   }
   energy -= C*win.getArea();
   return energy;
}

void
TreeNode1i::cmpBnds(){
   Res xx, yy;

   // Get a window in the range if there is
   if ((!findEnds(Res(winPair.win1.ul_x, winPair.win1.lr_x),
        Res(winPair.win2.ul_x, winPair.win2.lr_x),
        Res(szRange.wmin, szRange.wmax), &xx)) ||
      (!findEnds(Res(winPair.win1.ul_y, winPair.win1.lr_y),
        Res(winPair.win2.ul_y, winPair.win2.lr_y),
        Res(szRange.hmin, szRange.hmax), &yy))){

       lb = INF;
       ub = INF;
       return;
   }

   ubWin = Window(xx.x, yy.x, xx.y, yy.y);
   if ((ubWin.ul_x > ubWin.lr_x) ||
      (ubWin.ul_y > ubWin.lr_y)){
       ubWin = Window();
   }

   int *hist_ub = ubWin.getHist(integralIms, nHistBin);
   ub = cmpEnergy(hist_ub, hist_target, weights, nHistBin, ubWin, C);

   Window smallestWin = winPair.getSmallestWin();
   int *hist_min = smallestWin.getHist(integralIms, nHistBin);
   Window largestWin = winPair.getLargestWin();
   int *hist_max = largestWin.getHist(integralIms, nHistBin);


   lb = 0;
   for (int k=0; k < nHistBin; k++){
      if (hist_target[k] < hist_min[k]){
         lb += weights[k]*pow((double)(hist_target[k] - hist_min[k]),2);
      } else if (hist_target[k] > hist_max[k]){
         lb += weights[k]*pow((double)(hist_target[k] - hist_max[k]), 2);
      }
   }

   lb -= C*min(largestWin.getArea(), szRange.hmax*szRange.wmax);


   destroyVector<int>(hist_min, nHistBin);
   destroyVector<int>(hist_max, nHistBin);
   destroyVector<int>(hist_ub,  nHistBin);
}



//initialize static variables
int ***TreeNode1i::integralIms = NULL;
int *TreeNode1i::hist_target = NULL;
double *TreeNode1i::weights = NULL;
SizeRange TreeNode1i::szRange;
double TreeNode1i::C = 0;
int TreeNode1i::nHistBin = 0;



Window
findBox1i(int **kpAssign, int nKeyPnt, int imH, int imW, double *weights, int nHistBin,
		int *hist_target, double C, SizeRange szRange, double tolFac){

   printf("Inside the findBox1i ===========================\n");

   int ***integralIms = cmpIntegralIms1(kpAssign, nKeyPnt, imH, imW, nHistBin);
   TreeNode1i::integralIms = integralIms;
   TreeNode1i::szRange = szRange;
   TreeNode1i::C = C;
   TreeNode1i::nHistBin = nHistBin;
   TreeNode1i::weights = weights;
   TreeNode1i::hist_target = hist_target;

   Window biggestWin(0,0, imW-1, imH-1);
   TreeNode1i rootNode(WindowPair(biggestWin, biggestWin));

   TreeNode1i bestNode = m_branch_bound<TreeNode1i>(rootNode, tolFac);

   destroyArrayThree<int>(integralIms, imH, imW, nHistBin);
   return bestNode.ubWin;
}
