#ifndef M_TREENODE2A_H
#define M_TREENODE2A_H
#include "m_TreeNode2.h"

/**
 *
 */
class TreeNode2a:public TreeNode2{
public:
   static int ***integralIms1, ***integralIms2, nHistBin;
   static double *weights, C;

   TreeNode2a(WindowPair winPair1_, WindowPair winPair2_):TreeNode2(winPair1_, winPair2_){};
   TreeNode2a(){};
   void cmpBnds();
private:
   void cmpUb1();
   void cmpUb2(double knownBest);
   void cmpUb3(double knownBest);
};


/**
 * Simultaneously find similar rectangles in two images by minimizing the objective:
 *    \f$ \sum_i {w_i[|h^1_i  - h^2_i| - C(h^1_i + h^2_i)]} \f$
 * \param imAssign1, imAssign2: 2D arrays for histogram bin assignments, entries should be
 *       positive integers.
 * \param imH1, imW1, imH2, imW2: height and width of the images
 * \param weights: a 1-dim array for weights for histogram bins
 * \param nHistBin: # of histogram bins
 * \param C: tradeoff b/t having good match to the target histogram and the size of the box
 * \param tolFactor: tolerance factor for stopping criteria of branch and bound
 *       if tolFactor = 0, run BB until global optimality is known.
 *       the bigger tolFactor, the earlier BB terminates.
 * \param knownBest: the solution is known to be better than this value.
 *       branches with the lower bound less than this value is not explored.
 *       if no leaf node has energy smaller than this value, the method
 *       return an arbitrary solution.
 * \return Best (within tolerance) rectangles that minimizes the desired energy
 */
WindowPair findBoxes2a(int **imAssign1, int imH1, int imW1,
                      int **imAssign2, int imH2, int imW2,
                      double *binWeights, int nHistBin,
                      double C, double tolFactor, double knownBest);

#endif
