/*
 * m_TreeNode3.h
 *
 * Created on: May 7, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#ifndef M_TREENODE3_H_
#define M_TREENODE3_H_
#include <string>
#include "m_utils.h"

/**
 * class for violated event
 */
class VEvt{
public:
   int start, end, time;
   VEvt(){
      start = -1; end = -1; time = -1;
   }
   VEvt(int start_, int end_, int time_):
      start(start_), end(end_), time(time_){};

   void copy(VEvt *newVevt);

   //Compute the energy for a violated event
   //gtEvtStart: start of the gt event
   double getEnergy(NumPair gtEvt, double **integralIms, double *mu);
   string str();
   bool isSame(VEvt other);
};


class TreeNode3{
public:
   static double **integralIms;
   static NumPair gtEvt;   // GT event to detect
   static double *mu; //scaling for importance of truncated events

   NumPair startBnd; //bound for the starting point
   NumPair endBnd;   //bound for the ending point
   NumPair timeBnd;  //bound for the current time.

   double lb, ub;
   VEvt ubEvt;


   TreeNode3(){};
   TreeNode3(NumPair startBnd_, NumPair endBnd_, NumPair timeBnd_):
      startBnd(startBnd_), endBnd(endBnd_), timeBnd(timeBnd_){lb = 0; ub = 0;};
   void copy(TreeNode3 *newTN);


   // Split the TreeNode3 into 2 new one.
   // Split the dimension with biggest size.
   void split(TreeNode3 *left, TreeNode3 *right);
   void cmpBnds(); // compute bounds

   VEvt getShortestVEvt(); //get shortest event
   VEvt getLongestVEvt();  //get longest event

   NumPair getShortestTrGtSeg(); //get shortest truncated gt segment
   NumPair getLongestTrGtSeg();  //get longest truncated gt segment
   string str();
   bool isEmpty();
};


/**
 * Find the most violated constraint, truncated event supported.
 *    \f$ \f$
 * \param pixVals a row vector of length len.
 * \param len the length of the signal
 * \param gtEvent the ground truth event
 * \param tolFac tolerance factor for stopping
 * \return most violated event
 */
VEvt findBox3(double *pixVals, int len, NumPair gtEvent, double *mu, double tolFac, double &energy);
VEvt findBox3b(double *pixVals, int len, NumPair gtEvent, double *mu, double tolFac, double &energy);




#endif /* M_TREENODE3_H_ */
