/*
 * m_multiLabelKSeg.cpp
 *
 * Created on: Nov 29, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#include "m_multiLabelKSeg.h"
#include <iostream>
#include <limits>
#include <sys/times.h>


using namespace std;

double cmpMLKS_energy(vector<MLKS_Rslt> assgn, double **A){
   double energy = 0;
   for (unsigned int i = 0; i < assgn.size(); i++){
      for (int j = assgn[i].startIdx; j <= assgn[i].endIdx; j++) energy += A[assgn[i].label][j];
   }
   return energy;
}

vector<MLKS_Rslt> multiLabelKseg(double **A, int nLabel, int len, int nSeg, double *energy){
   MLKS_Entry ***tab = buildArrayThree<MLKS_Entry>(nSeg, nLabel, len);

   for (int h =0; h < nSeg; h++){
      for (int u= 0; u < nLabel; u++) tab[h][u][0].f = A[u][0];
   }

   // build the dynamic programming table
   for (int t=0; t < len -1; t++){
      for (int h=0; h < nSeg; h++){
         //Compute best alternative
         //First, find the least two values
         int minLabel  = -1;
         int minLabel2 = -1;
         double minVal = numeric_limits<double>::infinity();
         double minVal2 = numeric_limits<double>::infinity();;

         //find the least two elements
         for (int u=0; u < nLabel; u++){
            double f = tab[h][u][t].f;
            if (f < minVal){
               minVal2 = minVal;
               minLabel2 = minLabel;
               minVal = f;
               minLabel = u;
            } else if (f < minVal2){
               minVal2 = f;
               minLabel2 = u;
            }
         }
         //assign the best alternative values
         for (int u=0; u < nLabel; u++){
            if (u == minLabel){
               tab[h][u][t].f_bestAlt = minVal2;
               tab[h][u][t].bestAltLabel = minLabel2;
            } else {
               tab[h][u][t].f_bestAlt = minVal;
               tab[h][u][t].bestAltLabel = minLabel;
            }
         }

         //update next entries in the table
         for (int u=0; u < nLabel; u++){
            if ((h==0) || (tab[h][u][t].f <= tab[h-1][u][t].f_bestAlt)){
               tab[h][u][t+1].f = A[u][t+1] + tab[h][u][t].f;
               tab[h][u][t+1].prevDifIdx = tab[h][u][t].prevDifIdx;
               tab[h][u][t+1].prevDifLabel = tab[h][u][t].prevDifLabel;
            } else {
               tab[h][u][t+1].f = A[u][t+1] + tab[h-1][u][t].f_bestAlt;
               tab[h][u][t+1].prevDifIdx = t;
               tab[h][u][t+1].prevDifLabel = tab[h-1][u][t].bestAltLabel;
            }
         }

      }
   }

   //Find the label assignment of the last frame
   int minLabel = -1;
   double minVal = numeric_limits<double>::infinity();

   //find the label that yields the minimum energy
   for (int u=0; u < nLabel; u++){
      double f = tab[nSeg-1][u][len-1].f;
      if (f < minVal){
         minVal = f;
         minLabel = u;
      }
   }
   *energy = minVal;

   int curLabel = minLabel;
   int curIdx = len - 1;
   int nAllowedSeg = nSeg;
   MLKS_Entry curTabEntry = tab[nAllowedSeg-1][curLabel][curIdx];
   vector<MLKS_Rslt> rslt;
   while (1){
      MLKS_Entry curTabEntry = tab[nAllowedSeg-1][curLabel][curIdx];
      int nxtIdx = curTabEntry.prevDifIdx;
      rslt.push_back(MLKS_Rslt(nxtIdx+1, curIdx, curLabel));
      if (nxtIdx < 0) break;
      curIdx = nxtIdx;
      curLabel = curTabEntry.prevDifLabel;
      nAllowedSeg--;
   }
   destroyArrayThree<MLKS_Entry>(tab, nSeg, nLabel, len);
   return rslt;
}

int *multiLabelKseg2(double **A, int nLabel, int len, int nSeg,
      bool **segSwchInd, double *energy){

   MLKS_Entry2 ***tab = buildArrayThree<MLKS_Entry2>(nSeg, nLabel, len);

   for (int h =0; h < nSeg; h++){
      for (int u= 0; u < nLabel; u++) tab[h][u][0].f = A[u][0];
   }

   // build the dynamic programming table
   for (int t=0; t < len -1; t++){
      for (int h=0; h < nSeg; h++){
         //update next entries in the table
         for (int u=0; u < nLabel; u++){
            double minVal = numeric_limits<double>::infinity();
            int prevLabel = -1;
            for (int u2=0; u2 < nLabel; u2++){ //label of the second last elem
               double aux;
               if (!segSwchInd[u2][u]){ //not switching segment
                  aux = A[u][t+1] + tab[h][u2][t].f;
               } else if ((h>0) && segSwchInd[u2][u]) { //switching segment
                  aux = A[u][t+1] + tab[h-1][u2][t].f;
               } else {
                  continue;
               }
               if (minVal > aux){
                  minVal = aux;
                  prevLabel = u2;
               }
            }
            tab[h][u][t+1].f = minVal;
            tab[h][u][t+1].prevLabel = prevLabel;
         }
      }
   }

   //Find the label assignment of the last frame
   int minLabel = -1;
   double minVal = numeric_limits<double>::infinity();

   //find the label that yields the minimum energy
   for (int u=0; u < nLabel; u++){
      double f = tab[nSeg-1][u][len-1].f;
      if (f < minVal){
         minVal = f;
         minLabel = u;
      }
   }
   *energy = minVal;

   int curLabel = minLabel;
   int nAllowedSeg = nSeg;
   int *rslt = buildVector<int>(len);

   for (int t=len-1; t >=0; t--){
      rslt[t] = curLabel;
      int prevLabel = tab[nAllowedSeg-1][curLabel][t].prevLabel;
      if ((prevLabel != -1) && (segSwchInd[prevLabel][curLabel])) nAllowedSeg--;
      curLabel = prevLabel;
   }

   return rslt;
}

vector<MLKS_Rslt>
multiLabelKseg3(double **A, int nLabel, int len, int lmin, int lmax, double *energy){
   //compute integral images
   double **intIms = new double*[nLabel];
   for (int k=0; k < nLabel; k++) {
      intIms[k] =  new double[len + 1];
      intIms[k][0] = 0;
      cmpIntegralIms_1D_1(A[k], len, intIms[k] + 1);
   }

   //obj[u]: minimum energy if the len of segment is u.
   //we are interested in computing obj[len]
   double *obj   =  new double[len + 1]; // objective values
   int    *nxt   =  new int[len + 1];    // next change point
   int    *label =  new int[len + 1];    // label of last segment

   if ((len < lmin) || (lmax < lmin)) {
      *energy = INF;
      vector<MLKS_Rslt> rslt;
      return rslt;
   }

   obj[0] = 0;
   for (int u=1; u < lmin; u++) obj[u] = INF;

   double *svmVals = new double[nLabel];
   for (int u = lmin; u <= len; u++){
      int almax = min(lmax, u); //adjusted lmax.

      int rng = almax - lmin + 1;
      double *cmbCost = new double[rng];
      int *lb = new int[rng];
      for (int l=lmin; l <= almax; l++){
         for (int k=0; k < nLabel; k++){
            svmVals[k] = intIms[k][u] - intIms[k][u-l] ; // SVM value
         }
         int winningClass = max_element(svmVals, svmVals + nLabel) - svmVals;
         cmbCost[l-lmin] = obj[u-l] - svmVals[winningClass];
         lb[l-lmin] = winningClass;
      }

      int lIdx = min_element(cmbCost, cmbCost + rng) - cmbCost;
      obj[u] = cmbCost[lIdx];
      nxt[u] = u - lmin - lIdx;
      label[u] = lb[lIdx];
      delete [] lb;
      delete [] cmbCost;
   }

   *energy = obj[len];
   vector<MLKS_Rslt> rslt;
   int u = len;
   while (u > 0){
      rslt.push_back(MLKS_Rslt(nxt[u], u-1, label[u]));
      u = nxt[u];
   }

   //memory deallocation
   delete [] svmVals;
   delete [] obj;
   delete [] nxt;
   delete [] label;
   for (int k=0; k < nLabel; k++) delete[] intIms[k];
   delete [] intIms;

   return rslt;
}

vector<MLKS_Rslt>
MMCSeg(double **A, int nLabel, int len, int lmin, int lmax, double *energy){

   //compute integral images
   double **intIms = (double **) calloc(nLabel, sizeof(double *));
   for (int k=0; k < nLabel; k++) {
      intIms[k] = (double *) calloc(len+1, sizeof(double));
      intIms[k][0] = 0;
      cmpIntegralIms_1D_1(A[k], len, intIms[k] + 1);
   }

   //obj[u]: minimum energy if the len of segment is u.
   //we are interested in computing obj[len]
   double *obj   = (double *) calloc(len + 1, sizeof(double)); // objective values
   int    *nxt   = (int *) calloc(len + 1, sizeof(int));   // next change point
   int    *label = (int *) calloc(len + 1, sizeof(int));   // label of last segment


   if ((len < lmin) || (lmax < lmin)) {
      *energy = INF;
      vector<MLKS_Rslt> rslt;
      return rslt;
   }

   obj[0] = 0;
   for (int u=1; u < lmin; u++) obj[u] = INF;

   vector<pair<double, int> > svmVals(nLabel);
   for (int u = lmin; u <= len; u++){
      int almax = min(lmax, u); //adjusted lmax.

      int rng = almax - lmin + 1;
      double *cmbCost = new double[rng];
      int *lb = new int[rng];
      for (int l=lmin; l <= almax; l++){
         for (int k=0; k < nLabel; k++){
            svmVals[k].first = intIms[k][u-l] - intIms[k][u]; //negative of SVM value
            svmVals[k].second = k;
         }
         partial_sort(svmVals.begin(), svmVals.begin() + 2, svmVals.end());
         double fstMaxVal = - svmVals[0].first;
         double sndMaxVal = - svmVals[1].first;
         double xi_l = 1 - fstMaxVal + sndMaxVal;

         if (xi_l > 0) cmbCost[l-lmin] = obj[u-l] + xi_l;
         else cmbCost[l-lmin] = obj[u-l];

         lb[l-lmin] = svmVals[0].second;
      }

      int lIdx = min_element(cmbCost, cmbCost + rng) - cmbCost;
      obj[u] = cmbCost[lIdx];
      nxt[u] = u - lmin - lIdx;
      label[u] = lb[lIdx];
      delete [] lb;
      delete [] cmbCost;
   }


   *energy = obj[len];
   vector<MLKS_Rslt> rslt;
   int u = len;
   while (u > 0){
      rslt.push_back(MLKS_Rslt(nxt[u], u-1, label[u]));
      u = nxt[u];
   }


   //memory deallocation
   free(obj);
   free(nxt);
   free(label);
   for (int k=0; k < nLabel; k++) free(intIms[k]);
   free(intIms);

   return rslt;
}

double cmpMMCS_energy(double **A, int nLabel, int len, vector<MLKS_Rslt> assign){
   if (assign.size() == 0) return INF;


   //compute integral images
   double **intIms = (double **) calloc(nLabel, sizeof(double *));
   for (int k=0; k < nLabel; k++) {
      intIms[k] = (double *) calloc(len+1, sizeof(double));
      intIms[k][0] = 0;
      cmpIntegralIms_1D_1(A[k], len, intIms[k] + 1);
   }

   double energy = 0;
   vector<pair<double, int> > svmVals(nLabel);
   for (unsigned int i = 0; i < assign.size(); i++){
      int u = assign[i].endIdx + 1;
      int l = assign[i].endIdx - assign[i].startIdx + 1;

      for (int k=0; k < nLabel; k++){
         svmVals[k].first = intIms[k][u-l] - intIms[k][u]; //negative of SVM value
         svmVals[k].second = k;
      }

      double segVal = - svmVals[assign[i].label].first;

      partial_sort(svmVals.begin(), svmVals.begin() + 2, svmVals.end());
      double nxtMaxVal;
      if (assign[i].label == svmVals[0].second) nxtMaxVal = - svmVals[1].first;
      else nxtMaxVal = - svmVals[0].first;

      double xi = 1 - segVal + nxtMaxVal;
      if (xi > 0) energy += xi;
   }

   // memory deallocation
   for (int k=0; k < nLabel; k++) free(intIms[k]);
   free(intIms);

   return energy;
}

vector<MLKS_Rslt> randomSeg(int len, int nLabel, int lmin, int lmax, double *nValSeg_out){

   //nValidSeg[u]: number of valid segmentation for time series of length u.
   //we are interested in computing nValidSeg[len]
   //even though the number is integer, the integer range is limited
   //need to use double to store large value
   double *nValidSeg   = (double *) calloc(len + 1, sizeof(double)); // objective values

   //forward pass, compute nValidSeg[u] for u=1,...,len.

   vector<MLKS_Rslt> rslt;
   if ((len < lmin) || (lmax < lmin)) return rslt;

   nValidSeg[0] = 1;
   for (int u=1; u < lmin; u++) nValidSeg[u] = 0;


   for (int u = lmin; u <= len; u++){
      int almax = min(lmax, u); //adjusted lmax.
      for (int l=lmin; l <= almax; l++) nValidSeg[u] += nValidSeg[u- l];
   }

   // backward pass, draw a random segmentation, starting from the end
   // of the time series. The probability of the last segment has length l
   // is proportional to the number of valid segmentation of the signal with length (len - l).
   if (nValidSeg[len] == 0) return rslt;

   // pick a random number from 1 to nValidSeg[len]
   // this value uniquely defines the segmentation.
//   srand(time(NULL));

   double r =  (double) rand();
   *nValSeg_out = nValidSeg[len];
   r = (r/(double)(RAND_MAX))*nValidSeg[len];

   int u = len;
   while (u > 0){
      int almax = min(lmax, u);
      int l=lmin;
      for ( ;l <= almax; l++){
         if (r <= nValidSeg[u-l]) break;
         r = r - nValidSeg[u-l];
      }

      int label = rand() % nLabel;
      rslt.push_back(MLKS_Rslt(u-l, u-1, label));
      u = u-l;
   }

   //memory deallocation
   free(nValidSeg);
   return rslt;
}


vector<MLKS_Rslt> KmeansSeg(double **D, int d, int n,
      double **Cs, int nClass, int lmin, int lmax, bool shldNorm, double &energy){

   // compute integral image for D
   double **IntD = new double*[n+1];
   IntD[0] = new double[d];
   memset(IntD[0], 0, d*sizeof(double));
   for (int r=1; r <= n; r++){
      IntD[r] = new double[d];
      for (int c=0; c < d; c++) IntD[r][c] = IntD[r-1][c] + D[r-1][c];
   }


   //obj[u]: minimum energy if the len of segment is u.
   //we are interested in computing obj[len]
   double *obj   = new double[n+1]; // objective values
   int    *nxt   = new int[n+1];    // next change point
   int    *label = new int[n+1];    // label of last segment


   if ((n < lmin) || (lmax < lmin)) {
      energy = INF;
      vector<MLKS_Rslt> rslt;
      return rslt;
   }

   obj[0] = 0;
   for (int u=1; u < lmin; u++) obj[u] = INF;

   double *classE = new double[nClass];
   double *featV = new double[d];
   double *diffV = new double[d];
   for (int u = lmin; u <= n; u++){
      int almax = min(lmax, u); //adjusted lmax.

      int rng = almax - lmin + 1;
      double *cmbCost = new double[rng];
      int *lb = new int[rng];
      for (int l=lmin; l <= almax; l++){
         cblas_dcopy(d, IntD[u], 1, featV, 1); // featV = IntD[u]
         cblas_daxpy(d, -1, IntD[u-l], 1, featV, 1); // featV = IntD[u] - IntD[u-l];
         if (shldNorm){ // if should normalize for the length
            cblas_dscal(d, 1.0f/l, featV, 1); // featV = featV/l;
         }

         for (int k=0; k < nClass; k++){
            cblas_dcopy(d, featV, 1, diffV, 1); // diffV = featV;
            cblas_daxpy(d, -1, Cs[k], 1, diffV, 1); //diffV = featV - Cs[k];
            double nrm2 = cblas_dnrm2(d, diffV, 1);
            classE[k]  = nrm2*nrm2;
         }

         int bestClass = min_element(classE, classE + nClass) - classE;
         cmbCost[l-lmin] = obj[u-l] + classE[bestClass];
         lb[l-lmin] = bestClass;
      }

      int lIdx = min_element(cmbCost, cmbCost + rng) - cmbCost;
      obj[u] = cmbCost[lIdx];
      nxt[u] = u - lmin - lIdx;
      label[u] = lb[lIdx];
      delete [] lb;
      delete [] cmbCost;
   }

   energy = obj[n];
   vector<MLKS_Rslt> rslt;
   if (energy != INF){
      int u = n;
      while (u > 0){
         rslt.push_back(MLKS_Rslt(nxt[u], u-1, label[u]));
         u = nxt[u];
      }
   }

   //memory deallocation
   //deallocate data
   for (int r=0; r <=n; r++) delete [] IntD[r];
   delete [] IntD;
   delete [] obj;
   delete [] nxt;
   delete [] label;
   delete [] classE;
   delete [] featV;
   delete [] diffV;
   return rslt;
}

