#ifndef M_UTILS_H
#define M_UTILS_H

#include<string>
//#include "m_malloc.h"
#include <limits>


using namespace std;

#define INF numeric_limits<double>::infinity()
#define EPS 0.0000000001

/* Compute Integral image
 * im: 2D array of int. Each entry must be a POSITIVE integer
 * which is less than or equal to nHistBin
 * Compute Integral Images for histograms using dynamic programming.
 */
int*** cmpIntegralIms(int **im, int imH, int imW, int nHistBin);

/* Compute integaral images, one for each bin of the histogram
 * kpAssign: nKeyPnt*3 array of positive integers.
 * kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are [row#, col#, bin#] of a keypoint.
 * It is possible for two keypoints to to have same row# and col#. In other words,
 * is is fine for a pixel to be assigned to two histogram bins.
 * Compute integral images for histograms using dynamic programming.
 */
int*** cmpIntegralIms1(int **kpAssign, int nKeyPnt, int imH, int imW, int nHistBin);


/* Compute two integral images, one for bins with positive weights
 * one for bins with negative weights.
 * im: 2D array of int. Each entry must be a POSITIVE integer
 * which is less than or equal to nHistBin
 * Compute Integral Images for histograms using dynamic programming.
 */
double*** cmpIntegralIms2(int **im, int imH, int imW, double *weights, int nHistBin);

/* Compute two integral images, one for bins with positive weights
 * one for bins with negative weights.
 * kpAssign: nKeyPnt*3 array of positive integers.
 * kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are [row#, col#, bin#] of a keypoint.
 * It is possible for two keypoints to to have same row# and col#. In other words,
 * is is fine for a pixel to be assigned to two histogram bins.
 * Compute integral images for histograms using dynamic programming.
 */
double*** cmpIntegralIms3(int **kpAssign, int nKeyPnt, int imH, int imW, double *weights, int nHistBin);

double ***cmpIntegralIms3_helper(int & imH, int & imW, double **& posIm, double **& negIm);

/**
 * Compute two integral images, one for positive and one for negative
 * pixVals: 2D array of double.
 * Similar to cmpIntegralIms2 and cmpIntegralIms3 except for the input format
 */
double*** cmpIntegralIms4(double **pixVals, int imH, int imW);


/**
 * Compute two integral images, one for positive and one for negative
 * pixVals: 1D array of double.
 */
double** cmpIntegralIms_1D(double *pixVals, int len);


/**
 * Compute one integral image
 * pixVals: 1D array of double
 */
void cmpIntegralIms_1D_1(const double *pixVals, const int len, double *intIm);


/* Turn imAssign into keypoint assignment format*/
int** imAssign2kpAssign(int **imAssign, int imH, int imW);

/* Turn kpAssign into an image of scalar values */
double **kpAssign2pixVals(int **kpAssign, int nKeyPnt, int imH, int imW, double *weights);


/* The lower bound is obtained by minimize:
 * abs(h1 - h_target) - C*h1 with h1_min <= h1 <= h1_max
 * This assume 0 <= C <= 1
 */
double getLowerBnd1_helper(int h1_min, int h1_max, int h_target, double C);

/* The lower bound is obtained by minimize:
 * abs(h1 - h2) - C*(h1 + h2)
 * s.t. h1_min <= h1 <= h1_max and h2_min <= h2 <= h2_max
 * This assume 0 <= C <= 1 and h1_max <= h2_max;
 */
double getLowerBnd2_helper2(int h1_min, int h1_max, int h2_min, int h2_max, double C);

/* The lower bound is obtained by minimize:
 * abs(h1 - h2) - C*(h1 + h2)
 * s.t. h1_min <= h1 <= h1_max and h2_min <= h2 <= h2_max
 * This assume 0 <= C <= 1
 */
double getLowerBnd2_helper(int h1_min, int h1_max, int h2_min, int h2_max, double C);
double getLowerBnd1(int* hist1_min, int* hist1_max, int* hist_target, double *weights,
                    int nHistBin, double C);
double getLowerBnd2(int* hist1_min, int* hist1_max, int* hist2_min, int* hist2_max, double *weights,
                    int nHistBin, double C);

double getLowerBnd3(int *hist_min, int *hist_max, double *weights, int nHistBin);

double cmpEnergy(int* hist1, int* hist2, double *weights, int nHistBin, double C);
double cmpEnergy2(int *hist1, double *weights, int nHistBin);
double cmpEnergy3(int *hist1, double **B, int nHistBin, int nPC, double alpha);
double cmpEnergy4(int *hist1, double **B, int nHistBin, int nPC, double alpha);
double cmpEnergy5(int *hist1, int *hist2, int nHistBin, double alpha);


class Res{
public:
   int x, y;
   Res(int x_ =1, int y_ = 1):x(x_), y(y_){};
};


class Window{
public:
   int ul_x, ul_y, lr_x, lr_y; // upper left and lower right corner
   Window(int ul_x_, int ul_y_, int lr_x_, int lr_y_);
   Window();
   Window(const Window &win);
   Window(const Window &win, Res winRes);
   int* getHist(int*** integralIms, int nHistBin);
   int* getHist(int ***integralIms, int nHistBin, Res winRes);

   // get  the positive and negative score of a window
   double *getPosNegScore(double ***integralIms);

   int getMaxDim();
   int getArea();
   void copy(Window *newWin);
   void split(Window *left, Window *right);

   //check if the returned window is empty or not
   Window getSubWin(int x, int y, int xDev, int yDev);
   //No not check if the returned window is empty or not.
   Window getSubWin2(int x, int y, int xDev, int yDev);

   Window intersectWin(Window otherWin);
   Window unionWin(Window otherWin);

   bool operator ==(const Window& win);
   string str();
};


/* Contain 2 windows
 */
class WindowPair{
public:
   Window win1, win2;
   Window getSmallestWin();
   Window getLargestWin();
   Window getRandomWin();
   WindowPair(Window win1_, Window win2_);
   WindowPair();
   int getMaxDim();
   void copy(WindowPair *newWinPair);
   void split(WindowPair *left, WindowPair *right);
   WindowPair getSubWinPair(int x, int y, int xDev, int yDev);
   string str();
private:
   int getRandNum(int minNum, int maxNum);
};


class SizeRange{
public:
   int hmin, hmax, wmin, wmax;
   SizeRange(int hmin_, int hmax_, int wmin_, int wmax_):
      hmin(hmin_), hmax(hmax_), wmin(wmin_), wmax(wmax_){};
   SizeRange(){};
   string str();
};

/*
 * Pair of integer numbers
 */
class NumPair{
public:
   int l, u;
   NumPair(){l = -1; u= -1;};
   NumPair(int l_, int u_):l(l_), u(u_){};
   NumPair(const NumPair &np){
      l = np.l;
      u = np.u;
   }

   void split(NumPair *left, NumPair *right);
   NumPair intersectNP(NumPair np);
   NumPair unionNP(NumPair np);
   double F1Loss(NumPair np); // 1 - F1_score

   bool isEmpty();
   int getLen();
   double *getPosNegScore(double **integralIms);
   string str();
};


/* Find segment with 2 ends, the first end is in the range end1Range
 * the second end is in the range end2Range
 * The size of the segment is in the range szRange
 * If the segment exists, return true, and the segment is returned
 * in ends.
 */
bool findEnds(Res end1Range, Res end2Range, Res szRng, Res *ends);


template <class AType>
class TreeNodeCompare{
public:
   bool operator() (const AType& lhs, const AType& rhs){
      return (lhs.lb > rhs.lb);
   };
};


int **readImMatrix(char *fileName, int *imH_, int *imW_, int *nHistBin_);

bool isImAssignGood(int **imAssign, int imH, int imW);
bool isWithinTolFactor(double bestUb, double lb, double tolFactor);




#endif
